--------------------------------------------------------------------------------------------------------------------------
-- Vocal Characters 0.1.0.3
-- (C) 2012 あかつきみさき（みくちぃP）
-- 
-- このJobプラグインについて
--  調声作業を簡略化するJobプラグインです.
--  
-- インストール
--  VOCALOID3に追加するファイルは「VocalCharacters.lua」です.
--  VOCALOID3上でパスを設定する必要があります.
--  適当なディレクトリ(フォルダー)に置いて,VOCALOID3の「ジョブ→Jobプラグインを管理」から該当ディレクトリ(フォルダー)へパスを設定すれば完了です.
--  アンインストールはパスの削除,直接削除してください.
-- 
-- 動作環境
--  VOCALOID3 , VOCALOID3 Jobプラグインが動作する環境
-- 
-- 使用方法
--  一つのトラック内の一つ以上のノートを含むパートを選択し,「ジョブ→Jobプラグインを実行」から実行してください.
-- 
-- バージョン情報
--  2012/12/11 Ver0.1.0.3 Update 出力したテキストのオープン機能の削除.
--  2012/10/10 Ver0.1.0.2 Update プラグインIDが「Lyric Catch」と重複していた問題の修正.
--  2012/09/20 Ver0.1.0.1 Update 一部環境でしか動かないバグの修正. 歌詞が「あ行」始まり以外のときに更新エラーがでるバグの修正.
--  2012/08/19 Ver0.1.0.0 PreRelease 自動出力のLuaファイルの問題の修正. openingの対応. 分岐に入らない場合の処理の削除.
--------------------------------------------------------------------------------------------------------------------------

-- プラグインマニフェスト関数.
function manifest()
    myManifest = {
        name = "Vocal Characters",
        comment = "事前に作成した調声データから自動で調声作業を行います",
        author = "Misaki Akatsuki_102",
        pluginID = "{CB560CDD-348D-4018-8816-91B50F4F336D}",
        pluginVersion = "0.1.0.3",
        apiVersion = "3.0.0.1"
    }
    return myManifest
end

-- VOCALOID3 Jobプラグインスクリプトのエントリポイント.
function main(processParam, envParam)

-- 実行時に渡されたパラメータを取得します.
    local beginPosTick = processParam.beginPosTick	-- 選択範囲の始点時刻（ローカルTick）.
    local endPosTick   = processParam.endPosTick	-- 選択範囲の終点時刻（ローカルTick）.
    local songPosTick  = processParam.songPosTick	-- カレントソングポジション時刻（ローカルTick）.

-- 実行時に渡された実行環境パラメータを取得します.
    local scriptDir  = envParam.scriptDir	-- Luaスクリプトが配置されているディレクトリパス（末尾にデリミタ "\" を含む）.
    local scriptName = envParam.scriptName	-- Luaスクリプトのファイル名.
    local tempDir    = envParam.tempDir		-- Luaプラグインが利用可能なテンポラリディレクトリパス（末尾にデリミタ "\" を含む）.

    local noteEx     = {}
    local noteExList = {}
    local noteCount
    local retCode
    local myVelo
    local idx
    local sequencePath = VSGetSequencePath()

    local fileHelp = 'VC_Help.txt'
    local fileLua = 'VocalCharactersData.lua'
    
    -- luaファイルがあるかで生成を決める
    flug = io.open(fileLua,"r")
    
-- 初期設定
    if (flug == nil) then
--        VSMessageBox("file error!", 0)
        io.output(fileHelp)
        io.write('プログラミング言語「Lua」の法則にしたがって記述してください.\n' ..
                    '不安な場合は同梱のサンプルファイルを参考にしてください.\n' ..
                    '\n' ..
                    fileLua ..  'のデフォルトの内容は以下のようになっています.\n' ..
                    '\n' ..
                    'function ConvertVelocity(lyric)\n' ..
                    '  local velocity = 64\n' ..
                    '  if (lyric == "あ") then\n' ..
                    '    velocity = 64\n' ..
                    '--------------------[[ ここから ]]--------------------\n' ..
                    '  elseif (lyric == "い") then\n' ..
                    '    velocity = 64\n' ..
                    '  elseif (lyric == "う") then\n' ..
                    '    velocity = 64\n' ..
                    '--------------------[[ ここまで ]]--------------------\n' ..
                    '  else\n' ..
                    '    velocity = 64\n' ..
                    '  end\n' ..
                    '  return velocity\n' ..
                    'end\n\n' ..
                    'function ConvertOpening(lyric)\n' ..
                    '  local opening = 127\n' ..
                    '  if (lyric == "あ") then\n' ..
                    '    opening = 127\n' ..
                    '--------------------[[ ここから ]]--------------------\n' ..
                    '  elseif (lyric == "い") then\n' ..
                    '    opening = 127\n' ..
                    '  elseif (lyric == "う") then\n' ..
                    '    opening = 127\n' ..
                    '--------------------[[ ここまで ]]--------------------\n' ..
                    '  else\n' ..
                    '    opening = 127\n' ..
                    '  end\n' ..
                    '  return opening\n' ..
                    'end\n' ..
                    '\n' ..
                    'Vocal Charactersではこのデータを参照して自動調声を行います.このデータはユーザーが任意で調整してください.\n' ..
                    '\n' ..
                    'elseif (lyric == "ノートに入力されている歌詞") then\n' ..
                    '  velocity = 設定したいベロシティの値(0 から127の数字)\n' ..
                    '  \n' ..
                    '上の文を「ここから」「ここまで」の間に挿入してください.それ以外の場所に挿入するとうまく動作しません.\n' ..
                    '\n' ..
                    '一例(velocityの場合)\n' ..
                    '--------------------[[ ここから ]]--------------------\n' ..
                    '  elseif (lyric == "い") then\n' ..
                    '    velocity = 64\n' ..
                    '  elseif (lyric == "う") then\n' ..
                    '    velocity = 64\n' ..
                    '  elseif (lyric == "ん") then\n' ..
                    '    velocity = 0\n' ..
                    '--------------------[[ ここまで ]]--------------------\n' ..
                    '(lyric == "ノートに入力されている歌詞")の文でダブルクォーテーションで囲まれている部分は必ず囲うようにしてください.\n' ..
                    '一字一句でも間違うと,プログラムはうまく動作しません.(単語途中以外の空白・スペース・改行は無視されます)\n' ..
                    '\n※文字コードの取り扱いに注意してください.\n(BOMが付加されないようにしてください.)\n')
        io.close()
   
        io.output(fileLua)
        io.write('function ConvertVelocity(lyric)\n' ..
                    '  local velocity = 64\n' ..
                    '  if (lyric == "あ") then\n' ..
                    '    velocity = 64\n' ..
                    '--------------------[[ ここから ]]--------------------\n' ..
                    '  elseif (lyric == "い") then\n' ..
                    '    velocity = 64\n' ..
                    '  elseif (lyric == "う") then\n' ..
                    '    velocity = 64\n' ..
                    '--------------------[[ ここまで ]]--------------------\n' ..
                    '  else\n' ..
                    '    velocity = 64\n' ..
                    '  end\n' ..
                    '  return velocity\n' ..
                    'end\n\n' ..
                    'function ConvertOpening(lyric)\n' ..
                    '  local opening = 127\n' ..
                    '  if (lyric == "あ") then\n' ..
                    '    opening = 127\n' ..
                    '--------------------[[ ここから ]]--------------------\n' ..
                    '  elseif (lyric == "い") then\n' ..
                    '    opening = 127\n' ..
                    '  elseif (lyric == "う") then\n' ..
                    '    opening = 127\n' ..
                    '--------------------[[ ここまで ]]--------------------\n' ..
                    '  else\n' ..
                    '    opening = 127\n' ..
                    '  end\n' ..
                    '  return opening\n' ..
                    'end\n')
        io.close()
        
        --[[
        io.output('firstsetting.bat')
        io.write('@echo off\n' ..
                    'cd ' .. scriptDir .. '\n' ..
                    '\n' ..
                    'echo Open "' .. fileHelp .. ' & ' .. fileLua .. '"\n' ..
                    '\n' ..
                    'start notepad "' .. fileHelp .. '"\n' ..
                    'start notepad "' .. fileLua .. '"\n')
        io.close()
        
        if(VSMessageBox("このスクリプトと同じフォルダーに" .. fileHelp ..  ", " .. fileLua .. "を作成しました.\n" .. fileHelp ..  "を参考にし," .. fileLua ..  "に調声内容を記入してください.\n\nファイルをメモ帳で起動しますか?", 4) == 6) then
            os.execute(scriptDir .. 'firstsetting.bat')
        end
        ]]
    VSMessageBox("このスクリプトと同じフォルダーに" .. fileHelp ..  ", " .. fileLua .. "を作成しました.\n" .. fileHelp ..  "を参考にし," .. fileLua ..  "に調声内容を記入してください.\n\n※文字コードの取り扱いに注意してください.\n(BOMが付加されないようにしてください.)", 0)
    return 0

-- ファイルがある場合
    else
    -- 読み込み
    require('VocalCharactersData')

-- ノートを取得してノートイベント配列へ格納します.
    VSSeekToBeginNote()
    idx = 1
    retCode, noteEx, myVelo = VSGetNextNoteEx()
    
    while (retCode == 1) do
        noteExList[idx] = noteEx
        retCode, noteEx, myVelo = VSGetNextNoteEx()
        idx = idx + 1
    end

-- 読み込んだノートの総数.
    noteCount = table.getn(noteExList)
    if (noteCount == 0) then
        VSMessageBox("読み込んだノートがありません.", 0)
        return 0
    end
	
-- 取得したノートイベントを更新します.
    for idx = 1, noteCount do
        local updNoteEx = {}
        updNoteEx = noteExList[idx]

        updNoteEx.velocity = ConvertVelocity(noteExList[idx].lyric)
        --[[
        updNoteEx.bendDepth = 0
        updNoteEx.bendLength = 0
        updNoteEx.risePort = 1
        updNoteEx.fallPort = 1
        updNoteEx.decay = 100
        updNoteEx.accent = 100
        ]]
        updNoteEx.opening = ConvertOpening(noteExList[idx].lyric)
    --[[    
        updNoteEx.vibratoLength = 100
        updNoteEx.vibratoType = 1
        ]]
        retCode = VSUpdateNoteEx(updNoteEx);
        if (retCode ~= 1) then
            VSMessageBox("更新エラー発生!!", 0)
            return 1
        end
        end
    end

-- 正常終了.
    return 0
end

